/*****************************************************************************
 *  pptpdata.h    : $Revision:   1.21  $
 *  Modified      : $Date:   May 13 1996 14:48:38  $
 *  By            : $Author:   tmortsol  $, U.S. Robotics Access Corp.
 *****************************************************************************
 *          Copyright (c) $Date:   May 13 1996 14:48:38  $,                  *
 *            U.S. Robotics Access Corp., Microsoft Corp.                    *
 *                       All Rights Reserved                                 *
 *****************************************************************************
 *                          "PPTP MODULE"                                    *
 * This software may be used to implement the POINT to POINT TUNNELING       *
 * PROTOCOL as defined by the PPTP forum.  The PPTP forum consists of:       *
 * U.S. Robotics, Microsoft, Ascend Communications, 3Com/Primary Access,     *
 * Telematics.                                                               *
 *****************************************************************************
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain this entire header
 *    including the above copyright notice, this list of conditions and
 *    the following disclaimer.
 * 2. The name of U.S. Robotics may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission of U.S. Robotics.
 *
 * THIS SOFTWARE IS PROVIDED BY U.S. ROBOTICS ACESS CORP. ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL U.S. ROBOTICS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*END_HEADER*/

/*
 * pptpdata.h
 *
 * This header file contains data type structure definitions used by the
 * PPTP protocol engine and its interfaces
 */

#if !defined(_pptpdata_h)
#define _pptpdata_h

/*
 * PptpInternalTimerId - structure used to define internal timers
 */

typedef struct _PptpInternalTimer {
  BOOL timerScheduled;
  PptpCallbackFunction callbackFunction;
  PptpCallbackArgument callbackArgument;
  PptpMsec expirationTimeMsec;
  struct _PptpInternalTimer *prev;
  struct _PptpInternalTimer *next;
} PptpInternalTimer;

/*
 * PptpCallLineState - the states of a call line
 *
 * PPTP_CALL_LINE_NO_STATE
 *   The entry in the callLineDescriptorTable is currently not
 *   in use
 *
 * PPTP_CALL_LINE_ALLOCATED
 *   The entry in the callLineDescriptorTable has just been allocated
 *   to use for a call
 *
 * PPTP_CALL_LINE_WAITING_CONTROL_CHANNEL
 *   The call line is waiting for a control channel to become established
 *   before it can start call establishment
 *
 * PPTP_CALL_LINE_CALLING
 *   The call represents an outbound call (from the server's perspective)
 *   that is currently connecting
 *
 * PPTP_CALL_LINE_OFFERING
 *   The call represents an inbound call (from the server's perspective)
 *   that has just been detected but not answered
 *
 * PPTP_CALL_LINE_ANSWERING
 *   The call represents an inbound call (from the server's perspective)
 *   that has been answered and is currently connecting
 *
 * PPTP_CALL_LINE_CONNECTED
 *   The call is currently established
 *
 * PPTP_CALL_LINE_DISCONNECTING_LOCAL
 *   The call has been disconnected from the remote end but has not
 *   yet been disconnected at the local end
 *
 * PPTP_CALL_LINE_DISCONNECTING_REMOTE
 *   The call has been disconnected from the local end but has not
 *   yet been disconnected at the remote end
 *
 * PPTP_CALL_LINE_LOST
 *   The control channel for the call has been lost and the call is
 *   in the process of disconnecting at the local end
 */

typedef enum {
	PPTP_CALL_LINE_NO_STATE = 0,
	PPTP_CALL_LINE_ALLOCATED,
	PPTP_CALL_LINE_WAITING_CONTROL_CHANNEL,
	PPTP_CALL_LINE_CALLING,
	PPTP_CALL_LINE_OFFERING,
	PPTP_CALL_LINE_ANSWERING,
	PPTP_CALL_LINE_CONNECTED,
	PPTP_CALL_LINE_DISCONNECTING_LOCAL,
	PPTP_CALL_LINE_DISCONNECTING_REMOTE,
	PPTP_CALL_LINE_LOST_CONTROL_CHANNEL,
	PPTP_CALL_LINE_NUMBER_STATES
} PptpCallLineState;

typedef struct {
  UCHAR *data;
  ULONG dataLength;
  PptpSeqNumber seqNumber;
  void *completionArg;
} PptpDataPacketDescriptor;

typedef struct _PptpCallLineDescriptor {
#if defined(PPTP_SYMMETRIC_SERVER)
	BOOL			symmetricCallReceived;
#endif /* defined(PPTP_SYMMETRIC_SERVER) */
	PptpCallLineState	callLineState;
	struct _PptpControlChannelDescriptor
				*controlChannelDescriptorp;
	struct _PptpControlChannelSeekDescriptor
				*controlChannelSeekDescriptorp;
	PptpControlPacket       pendingControlPacket;
	PptpReturnCode
		(*sendCallRequestFunction)
		(struct _PptpCallLineDescriptor *deviceDescriptorp);
	PptpCallLineNumber	localCallLineNumber;
	PptpCallLineNumber	remoteCallLineNumber;
	PptpCallDeviceNumber	callDeviceNumber;
	PptpCallSerialNumber	callSerialNumber;
	PptpPhysChannelID	physChannelID;
	PptpBPS			connectBPS;
	PptpCallBearerType	callBearerType;
	PptpFramingType		framingType;
	PptpPacketWindow	localRxPacketWindow;
	PptpPacketWindow	remoteRxPacketWindow;
	PptpPacketWindow	remoteRxPacketWindowMax;
	PptpPacketProcDelay	localRxPacketProcDelay;
	PptpPacketProcDelay	remoteRxPacketProcDelay;
	PptpDataPacketDescriptor
				recvDataPacketArray
				  [PPTP_MAX_PACKET_WINDOW + 1];
	ULONG				recvDataPacketHeadIndex;
	ULONG				recvDataPacketPassedOnIndex;
	ULONG				recvDataPacketTailIndex;
	PptpSeqNumber		lastOutputSeqNumber;
	PptpAckNumber		highestAckedOutputSeqNumber;
	PptpSeqNumber		highestInputSeqNumber;
	PptpSeqNumber		highestProcessedInputSeqNumber;
	PptpSeqNumber		highestAckedInputSeqNumber;
	ULONG				numberDiscardedRecvPackets;
	PptpInternalTimer	flowControlTimer;
	PptpInternalTimer	delayedAckTimer;
	BOOL			localDataChannelFlowControlOn;
	BOOL			remoteDataChannelFlowControlOn;
	PptpLock		lock;
} PptpCallLineDescriptor;

typedef enum {
	PPTP_CONTROL_CHANNEL_NO_STATE = 0,
	PPTP_CONTROL_CHANNEL_ALLOCATED,
	PPTP_CONTROL_CHANNEL_CONNECTING,
	PPTP_CONTROL_CHANNEL_STARTING_SESSION,
	PPTP_CONTROL_CHANNEL_ESTABLISHED,
	PPTP_CONTROL_CHANNEL_STOPPING_SESSION,
	PPTP_CONTROL_CHANNEL_DISCONNECTING,
	PPTP_CONTROL_CHANNEL_LOST,
	PPTP_CONTROL_CHANNEL_NUMBER_STATES
} PptpControlChannelState;

typedef enum {
	PPTP_CONTROL_CHANNEL_INACTIVITY_NO_STATE = 0,
	PPTP_CONTROL_CHANNEL_INACTIVITY_IDLE_STATE,
	PPTP_CONTROL_CHANNEL_INACTIVITY_WAITING_ECHO_RESPONSE_STATE
} PptpControlChannelInactivityState;

typedef struct {
	PptpControlPacket	currentPacket;
	ULONG			currentPacketLength;
} PptpControlTCPDescriptor;

typedef struct _PptpControlChannelDescriptor {
	PptpControlChannelState		controlChannelState;
	PptpControlChannelIndex		controlChannelIndex;
	BOOL				locallyInitiatedConnection;
    PptpRemoteNetConfigEntry	remoteNetConfigEntry;
	PptpEndpointHandle		controlChannelHandle;
	PptpControlTCPDescriptor	controlTCPDescriptor;
	PptpControlChannelInactivityState controlChannelInactivityState;
	PptpEchoIdentNumber		echoIdentNumber;
	PptpInternalTimer		inactivityTimer;
	PptpLock			lock;
} PptpControlChannelDescriptor;

typedef enum {
	PPTP_SEEK_NO_STATE,
	PPTP_SEEKING_OUTSTANDING_CHANNEL,
	PPTP_SEEKING_NEW_CHANNEL
} PptpControlChannelSeekState;

typedef struct _PptpControlChannelSeekDescriptor {
	PptpControlChannelSeekState	controlChannelSeekState;
	PptpControlChannelSeekIndex	controlChannelSeekIndex;
	PptpAddressListIndex		currentAddressListIndex;
	PptpAddressListIndex		finalAddressListIndex;
	PptpAddressListIndex		lastEstablishedAddressListIndex;
	PptpAddressList			addressList;
	PptpControlChannelDescriptor	*pendingControlChannelDescriptorp;
} PptpControlChannelSeekDescriptor;

typedef struct _PptpDataDescriptor{
	PptpCallLineNumber	callLineNumber;
	BOOLEAN				seqNumberPresent;
	BOOLEAN				ackNumberPresent;
	PptpSeqNumber		seqNumber;
	PptpAckNumber		ackNumber;
	UCHAR				*data;
	ULONG				dataLength;
	void				*completionArg;
} PptpDataDescriptor;

typedef struct {
  PptpConfig config;
  struct {
    PptpCallLineState		callLineState;
    PptpControlChannelIndex	controlChannelIndex;
    PptpControlChannelSeekIndex	controlChannelSeekIndex;
    PptpCallLineNumber		localCallLineNumber;
    PptpCallLineNumber		remoteCallLineNumber;
    PptpCallSerialNumber	callSerialNumber;
    PptpPhysChannelID		physChannelID;
    PptpBPS			connectBPS;
    PptpCallBearerType		callBearerType;
    PptpFramingType		framingType;
    PptpPacketWindow		localRxPacketWindow;
    PptpPacketWindow		remoteRxPacketWindow;
    PptpPacketWindow		remoteRxPacketWindowMax;
    PptpPacketProcDelay		localRxPacketProcDelay;
    PptpPacketProcDelay		remoteRxPacketProcDelay;
    ULONG systemSpecificCallLineState;
  } callLineInfo[PPTP_MAX_CALL_LINE_NUMBER];
  struct {
    PptpControlChannelState	controlChannelState;
    BOOL			locallyInitiatedConnection;
    PptpRemoteNetConfigEntry	remoteNetConfigEntry;
  } controlChannelInfo[PPTP_MAX_CONTROL_CHANNEL_NUMBER];
  struct {
    PptpControlChannelSeekState controlChannelSeekState;
    PptpControlChannelIndex	controlChannelIndex;
    PptpAddressListIndex	currentAddressListIndex;
    PptpAddressListIndex	finalAddressListIndex;
    PptpAddressListIndex	lastEstablishedAddressListIndex;
  } controlChannelSeekInfo[PPTP_MAX_CONTROL_CHANNEL_SEEK_NUMBER];
} PptpDebugStateInfo;

#if defined(PPTP_REDUCE_STATIC_STORAGE)

typedef struct {
  PptpConfig currentPptpConfig;
  PptpControlChannelDescriptor
    controlChannelDescriptorTable[PPTP_MAX_CONTROL_CHANNEL_NUMBER];
  PptpCallLineDescriptor
    callLineDescriptorTable[PPTP_MAX_CALL_LINE_NUMBER];
  PptpControlChannelSeekDescriptor
    controlChannelSeekDescriptorTable
      [PPTP_MAX_CONTROL_CHANNEL_SEEK_NUMBER];
  PptpEndpointHandle listenControlEndpointHandle;
  PptpEndpointHandle dataChannelEndpointHandle;
#if defined(PPTP_USE_STRONG_LOCKS)
  PptpLock pptpLockStrong;
#else
  PptpLock pptpLockGlobal;
#endif /* defined(PPTP_USE_STRONG_LOCK) */
  PptpExternalTimerId externalTimerId;
  PptpInternalTimer *internalTimerHead;
  PptpInternalTimer *internalTimerTail;
} PptpGlobals;

extern PptpGlobals *PptpGp;

#define PPTP_GP(var) (PptpGp->var)

#define currentPptpConfig PPTP_GP(currentPptpConfig)
#define controlChannelDescriptorTable PPTP_GP(controlChannelDescriptorTable)
#define callLineDescriptorTable PPTP_GP(callLineDescriptorTable)
#define controlChannelSeekDescriptorTable \
PPTP_GP(controlChannelSeekDescriptorTable)

#if defined(PPTP_USE_STRONG_LOCKS)
#define g_PptpLockStrong PPTP_GP(pptpLockStrong)
#else
#define g_PptpLockGlobal PPTP_GP(pptpLockGlobal)
#endif /* defined(PPTP_USE_STRONG_LOCK) */

#else

//extern PptpConfig currentPptpConfig;
//extern PptpControlChannelDescriptor
//controlChannelDescriptorTable[PPTP_MAX_CONTROL_CHANNEL_NUMBER];
//extern PptpCallLineDescriptor
//deviceDescriptorTable[PPTP_MAX_CALL_LINE_NUMBER];
//extern PptpControlChannelSeekDescriptor
//controlChannelSeekDescriptorTable[PPTP_MAX_CONTROL_CHANNEL_SEEK_NUMBER];

extern PptpConfig *currentPptpConfig;
extern PptpControlChannelDescriptor *controlChannelDescriptorTable;
extern PptpCallLineDescriptor *deviceDescriptorTable;
extern PptpControlChannelSeekDescriptor *controlChannelSeekDescriptorTable;

#if defined(PPTP_USE_STRONG_LOCKS)
extern PptpLock g_PptpLockStrong;
#else
extern PptpLock g_PptpLockGlobal;
#endif /* defined(PPTP_USE_STRONG_LOCKS) */

#endif /* defined(PPTP_REDUCE_STATIC_STORAGE) */

#define assertOnFatalError currentPptpConfig->AssertOnFatalError
#define assertOnProtocolError currentPptpConfig->AssertOnProtocolError
#define assertOnMemoryAllocationFailure currentPptpConfig->AssertOnMemoryAllocationFailure

#endif /* !defined(_pptpdata_h) */
